// TestApp.cs
//
// This little application creates two threads and then uses
// the ThreadWaitHandle class (see ThreadWaitHandle.cs) to
// perform a blocking wait until both of those threads have
// exited.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Threading;
using System.Runtime.InteropServices;
using System.Windows.Forms;

class App
{
    static void Main()
    {
        ThreadStart threadProc = new ThreadStart(ThreadProc);
        Thread t1 = new Thread(threadProc);
        t1.Start();
        Thread t2 = new Thread(threadProc);
        t2.Start();

        WaitHandle[] threadHandles = {new ThreadWaitHandle(t1), new ThreadWaitHandle(t2)};

        Console.WriteLine("Waiting for threads to exit:");
        Console.WriteLine("  0x{0:x8}", threadHandles[0].Handle.ToInt32());
        Console.WriteLine("  0x{0:x8}", threadHandles[1].Handle.ToInt32());

        bool waitSucceeded = WaitHandle.WaitAll(threadHandles);
        Console.WriteLine("Done waiting (result = {0})", waitSucceeded);
    }

    static void ThreadProc()
    {
        // Display the native HANDLE along with the native thread ID
        // (just so a program like sysinternals.com's process explorer
        // can be used to verify the thread handle being displayed.
        //
        string msg =
            string.Format( "Managed handle = 0x{0:x8}, native TID = {1}.  Press OK to exit thread.",
                           ThreadWaitHandle.GetNativeHandle().ToInt32(), GetCurrentThreadId() );
        MessageBox.Show(msg, "ThreadProc");
    }

    [ DllImport("kernel32.dll") ]
    static extern int GetCurrentThreadId();
}